@@ -160,12 +160,14 @@ EventHistory.frame <- function(formula,
         if (!(formula.names[1]=="~")
             ||
             (match("$",formula.names,nomatch=0)+match("[",formula.names,nomatch=0)>0)){
-            stop("Invalid specification of formula. Perhaps forgotten right hand side?\nNote that any subsetting, ie data$var or data[,\"var\"], is invalid for this function.")}
+            stop("Invalid specification of formula. Perhaps forgotten right hand side?\nNote that any subsetting, i.e., data$var or data[,\"var\"], is not supported here.")}
         else{
-            if (!(any(match(c("survival::Surv","Surv","prodlim::Hist","Hist"),
-                            formula.names,nomatch=0))))
-                stop("formula is NOT a proper survival formula,\nwhich must have a `Surv' or `Hist' object as response.")
-        }
+                if (!(any(match(c("survival::Surv","Surv","prodlim::Hist","Hist"),
+                                formula.names,nomatch=0)))
+                    && !inherits(data[[formula.names[[2]]]],"Surv")
+                    && !inherits(data[[formula.names[[2]]]],"Hist"))
+                    stop("formula is NOT a proper survival formula,\nwhich must have a `Surv' or `Hist' object as response.")
+            }
     }
     # }}}
     # {{{call model.frame
@@ -184,44 +186,46 @@ EventHistory.frame <- function(formula,
     # {{{ get all variables and remove missing values
     ## use the stripped formula because, otherwise
     ## it may be hard to know what variables are, e.g.,
-    ## FGR uses cov2(var,tf=qfun) where qfun is a function 
-    mm <- na.omit(get_all_vars(formula(Terms),data))
+    ## FGR uses cov2(var,tf=qfun) where qfun is a function
+    mm <- na.omit(get_all_vars(formula(Terms),data=data))
     if (NROW(mm) == 0) stop("No (non-missing) observations")
     # }}}
 
     # {{{ extract response
     if (response==TRUE && attr(Terms,"response")!=0){
-        event.history <- model.response(model.frame(update(formula,".~1"),
-                                                    data=mm))
+        event.history <- model.response(model.frame(update(formula,".~1"),data=mm))
         # }}}
         # {{{ Fix for those who use `Surv' instead of `Hist' 
-        if (match("Surv",class(event.history),nomatch=0)!=0){
-            attr(event.history,"model") <- "survival"
-            attr(event.history,"cens.type") <- "rightCensored"
-            attr(event.history,"entry.type") <- ifelse(ncol(event.history)==2,"","leftTruncated")
-            if (attr(event.history,"entry.type")=="leftTruncated")
-                colnames(event.history) <- c("entry","time","status")
-        }
-        # }}}
-    }else event.history <- NULL
-    # {{{ design
-    design <- model.design(Terms,
-                           data=mm,
-                           maxOrder=1,
-                           dropIntercept=dropIntercept,
-                           unspecialsDesign=unspecialsDesign,
-                           specialsFactor=specialsFactor,
-                           specialsDesign=specialsDesign)
+        if (inherits(event.history,"Surv")
+            || inherits(data[[formula.names[[2]]]],"Surv")){
+                event.history = as.matrix(event.history)
+                class(event.history) = "Hist"
+                attr(event.history,"model") <- "survival"
+                attr(event.history,"cens.code") <- 0
+                attr(event.history,"cens.type") <- "rightCensored"
+                attr(event.history,"entry.type") <- ifelse(ncol(event.history)==2,"","leftTruncated")
+                if (attr(event.history,"entry.type")=="leftTruncated")
+                    colnames(event.history) <- c("entry","time","status")
+            }
     # }}}
-    out <- c(list(event.history=event.history),
-             design[sapply(design,length)>0])
-    attr(out,"Terms") <- Terms
-    attr(out,"na.action") <- attr(mm,"na.action")
-    class(out) <- "EventHistory.frame"
-    out
+}else event.history <- NULL
+# {{{ design
+design <- model.design(Terms,
+                       data=mm,
+                       maxOrder=1,
+                       dropIntercept=dropIntercept,
+                       unspecialsDesign=unspecialsDesign,
+                       specialsFactor=specialsFactor,
+                       specialsDesign=specialsDesign)
+# }}}
+out <- c(list(event.history=event.history),
+         design[sapply(design,length)>0])
+attr(out,"Terms") <- Terms
+attr(out,"na.action") <- attr(mm,"na.action")
+class(out) <- "EventHistory.frame"
+out
 }
-##' @S3method as.data.frame EventHistory.frame
-##' @method as.data.frame EventHistory.frame
+##' @export 
 as.data.frame.EventHistory.frame <- function(x,...){
     Y <- data.frame(unclass(x$event.history))
     X <- do.call("cbind",x[-1])
